"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BrailleStream = void 0;
const EncodingCategory_1 = require("../Common/EncodingCategory");
const BrailleData_1 = require("./BrailleData");
const BrailleEncoding_1 = require("./BrailleEncoding");
class BrailleStream {
    _chars = [];
    _currentStr = '';
    _processPosition = 0;
    _numberMode = false;
    constructor(chars) {
        if (chars) {
            this._chars = Array.from(chars);
        }
        this.invalidate();
    }
    get chars() {
        return this._chars;
    }
    set chars(value) {
        this.clear();
        for (const ch of value) {
            this._chars.push(ch);
        }
    }
    append(ch) {
        this._chars.push(ch.valueOf());
    }
    clear() {
        this._chars.length = 0;
        this.invalidate();
    }
    backspace() {
        this._chars.pop();
        this.invalidate();
    }
    space() {
        this._chars.push(BrailleEncoding_1.BrailleEncoding.None);
    }
    toString() {
        this.update();
        return this._currentStr;
    }
    invalidate() {
        this._currentStr = '';
        this._processPosition = 0;
        this._numberMode = false;
    }
    update() {
        while (this._processPosition < this._chars.length) {
            const ch = this._chars[this._processPosition];
            if (ch === BrailleEncoding_1.BrailleEncoding.None) {
                this._numberMode = false;
                this._currentStr += ' ';
            }
            else if (ch === BrailleEncoding_1.BrailleEncoding.FormattingNumber) {
                this._numberMode = true;
                this._currentStr += '#';
            }
            else {
                const category = EncodingCategory_1.EncodingCategory.Punctuation |
                    (this._numberMode
                        ? EncodingCategory_1.EncodingCategory.Number
                        : EncodingCategory_1.EncodingCategory.Letter);
                const exact = BrailleData_1.BrailleData.instance.lookup(ch, category).exact;
                if (exact.length > 0) {
                    this._currentStr += exact[0].toString();
                }
            }
            this._processPosition++;
        }
    }
}
exports.BrailleStream = BrailleStream;
//# sourceMappingURL=BrailleStream.js.map