"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CaesarUtils = void 0;
const CharacterConversion_1 = require("../Conversion/CharacterConversion");
class CaesarUtils {
    static rotateLetter(ch, rot) {
        if (ch.length !== 1) {
            throw new Error('Expected a single character');
        }
        rot %= this.alphaLength;
        if (rot < 0) {
            rot += this.alphaLength;
        }
        const code = ch.charCodeAt(0);
        let base = 0;
        if (this.isLowerAlpha(code)) {
            base = this.lowerA;
        }
        else if (this.isUpperAlpha(code)) {
            base = this.upperA;
        }
        else {
            return ch;
        }
        return String.fromCharCode(((code - base + rot) % this.alphaLength) + base);
    }
    static rotateLetterWithKey(ch, key, keyIndex, decrypt = false) {
        if (key.length === 0) {
            return ch;
        }
        const keyCh = this.getCharacterWithMod(key, keyIndex);
        let rot = CharacterConversion_1.CharacterConversion.toOrdinal(keyCh) - 1;
        if (decrypt) {
            rot = -rot;
        }
        return this.rotateLetter(ch, rot);
    }
    static isAlpha(ch) {
        const code = ch.charCodeAt(0);
        return this.isUpperAlpha(code) || this.isLowerAlpha(code);
    }
    static upperA = 'A'.charCodeAt(0);
    static upperZ = 'Z'.charCodeAt(0);
    static lowerA = 'a'.charCodeAt(0);
    static lowerZ = 'z'.charCodeAt(0);
    static alphaLength = 26;
    static getCharacterWithMod(str, index) {
        return str.charAt(index % str.length);
    }
    static isUpperAlpha(code) {
        return code >= this.upperA && code <= this.upperZ;
    }
    static isLowerAlpha(code) {
        return code >= this.lowerA && code <= this.lowerZ;
    }
}
exports.CaesarUtils = CaesarUtils;
//# sourceMappingURL=CaesarUtils.js.map