"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CharacterAutoConvert = void 0;
const CharacterEncoding_1 = require("./CharacterEncoding");
class CharacterAutoConvert {
    // Forced encoding can be useful when converting an entire string
    // or simply dealing with binary which has no leading zeros
    static convertCharacter(input, forcedCharacterEncoding) {
        let encoding = null;
        if (!forcedCharacterEncoding) {
            encoding = this.determineCharacterEncoding(input);
        }
        else {
            encoding = forcedCharacterEncoding;
        }
        if (encoding === CharacterEncoding_1.CharacterEncoding.None) {
            return '';
        }
        // Assume that Latin characters should remain
        if (encoding === CharacterEncoding_1.CharacterEncoding.Latin) {
            return input;
        }
        const baseTen = Number.parseInt(input, 10);
        if (encoding === CharacterEncoding_1.CharacterEncoding.Ascii) {
            return CharacterAutoConvert.asciiPrintable(baseTen);
        }
        const asciiOffset = 64;
        if (encoding === CharacterEncoding_1.CharacterEncoding.Ordinal) {
            return CharacterAutoConvert.asciiPrintable(baseTen + asciiOffset);
        }
        const binary = Number.parseInt(input, 2);
        if (encoding === CharacterEncoding_1.CharacterEncoding.FiveBitBinary &&
            this.appearsBinary(input)) {
            return CharacterAutoConvert.asciiPrintable(binary + asciiOffset);
        }
        if (encoding === CharacterEncoding_1.CharacterEncoding.EightBitBinary &&
            this.appearsBinary(input)) {
            return CharacterAutoConvert.asciiPrintable(binary);
        }
        if (encoding === CharacterEncoding_1.CharacterEncoding.Ternary) {
            const ternary = Number.parseInt(input, 3);
            return CharacterAutoConvert.asciiPrintable(ternary + asciiOffset);
        }
        return '';
    }
    static asciiPrintable(index) {
        if (index < 32 || index > 126) {
            return '';
        }
        return String.fromCharCode(index);
    }
    static determineCharacterEncoding(input) {
        if (input.match(/[a-z]/i)) {
            return CharacterEncoding_1.CharacterEncoding.Latin;
        }
        const numeric = Number.parseInt(input, 10);
        if (this.appearsBinary(input)) {
            if (input.length === 5) {
                return CharacterEncoding_1.CharacterEncoding.FiveBitBinary;
            }
            else if (input.length === 8 || input.length === 7) {
                return CharacterEncoding_1.CharacterEncoding.EightBitBinary;
            }
        }
        if (this.appearsTernary(input)) {
            return CharacterEncoding_1.CharacterEncoding.Ternary;
        }
        if (input.length < 3 && numeric > 0 && numeric < 27) {
            return CharacterEncoding_1.CharacterEncoding.Ordinal;
        }
        if (numeric > 64 && numeric < 91) {
            return CharacterEncoding_1.CharacterEncoding.Ascii;
        }
        if (numeric > 97 && numeric < 123) {
            return CharacterEncoding_1.CharacterEncoding.Ascii;
        }
        return CharacterEncoding_1.CharacterEncoding.None;
    }
    static BINARY_REGEX = new RegExp('^[01]+$');
    static appearsBinary(str) {
        return this.BINARY_REGEX.test(str);
    }
    static TERNARY_REGEX = new RegExp('^[0-2]{3}$');
    static appearsTernary(str) {
        return this.TERNARY_REGEX.test(str);
    }
}
exports.CharacterAutoConvert = CharacterAutoConvert;
//# sourceMappingURL=CharacterAutoConvert.js.map