"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SignificantFigures = void 0;
class SignificantFigures {
    static ceil(num, sigFigs) {
        return this.roundInternal(Math.ceil, num, sigFigs);
    }
    static floor(num, sigFigs) {
        return this.roundInternal(Math.floor, num, sigFigs);
    }
    static round(num, sigFigs) {
        return this.roundInternal(Math.round, num, sigFigs);
    }
    static roundInternal(func, num, sigFigs) {
        if (num === 0) {
            return num;
        }
        if (sigFigs <= 0) {
            throw new RangeError('sigFigs value must be positive');
        }
        const factor = this.getFactor(num, sigFigs);
        if (factor > 0) {
            return func(num / factor) * factor;
        }
        else if (factor < 0) {
            const absFactor = Math.abs(factor);
            return func(num * absFactor) / absFactor;
        }
        else {
            return num;
        }
    }
    static getFactor(num, sigFigs) {
        const absNum = Math.abs(num);
        let digits = 0;
        if (absNum < 1) {
            digits = -sigFigs;
        }
        else {
            digits = Math.ceil(Math.log(absNum) / Math.log(10)) - sigFigs;
        }
        if (digits < 0) {
            return -Math.pow(10, Math.abs(digits));
        }
        else {
            return Math.pow(10, digits);
        }
    }
}
exports.SignificantFigures = SignificantFigures;
//# sourceMappingURL=SignificantFigures.js.map