"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MorseCharacter = void 0;
const EncodingCategory_1 = require("../Common/EncodingCategory");
const EncodingCharacterBase_1 = require("../Common/EncodingCharacterBase");
const Helpers_1 = require("../Common/Helpers");
const MorseData_1 = require("./MorseData");
const MorseEncoding_1 = require("./MorseEncoding");
const MORSE_BITMASK = MorseEncoding_1.MorseEncoding.Dot | MorseEncoding_1.MorseEncoding.Dash;
class MorseCharacter extends EncodingCharacterBase_1.EncodingCharacterBase {
    static DOT = '.';
    static DASH = '-';
    // Character which may not appear in morse and is reserved for use by this class
    static RESERVED_DIVIDER = 'A';
    static toMorseString(encoding) {
        let morseChars = '';
        while (encoding !== MorseEncoding_1.MorseEncoding.None) {
            if ((encoding & MORSE_BITMASK) === MorseEncoding_1.MorseEncoding.Dot) {
                morseChars += MorseCharacter.DOT;
            }
            else if ((encoding & MORSE_BITMASK) === MorseEncoding_1.MorseEncoding.Dash) {
                morseChars += MorseCharacter.DASH;
            }
            else {
                throw new Error('Invalid morse bits');
            }
            encoding >>>= 2;
        }
        return morseChars;
    }
    static parseMorseString(morse) {
        let bits = MorseEncoding_1.MorseEncoding.None;
        for (let i = morse.length - 1; i >= 0; i--) {
            const ch = morse[i];
            if (ch === MorseCharacter.DOT) {
                bits |= MorseEncoding_1.MorseEncoding.Dot;
            }
            else if (ch === MorseCharacter.DASH) {
                bits |= MorseEncoding_1.MorseEncoding.Dash;
            }
            else {
                throw new Error('Invalid morse character');
            }
            bits <<= 2;
        }
        bits >>>= 2;
        return bits;
    }
    _morse;
    constructor(str = '', category = EncodingCategory_1.EncodingCategory.All) {
        super(MorseData_1.MorseData.instance, category);
        this._morse = str;
    }
    get morseString() {
        return this._morse;
    }
    set morseString(value) {
        this._morse = value;
        this.invalidateLookup();
    }
    backspace() {
        if (this._morse.length > 0) {
            this._morse = this._morse.substring(0, this._morse.length - 1);
            this.invalidateLookup();
        }
    }
    dot() {
        this._morse += MorseCharacter.DOT;
        this.invalidateLookup();
    }
    dash() {
        this._morse += MorseCharacter.DASH;
        this.invalidateLookup();
    }
    invertDotsAndDashes() {
        // Replace dots with a placeholder, dashes with dots, then placeholders with dashes
        Helpers_1.Helpers.assert(this._morse.indexOf(MorseCharacter.RESERVED_DIVIDER) < 0);
        this._morse = this._morse
            .replace(/\./g, 'A')
            .replace(/-/g, MorseCharacter.DOT)
            .replace(/A/g, MorseCharacter.DASH);
    }
    reverse() {
        this._morse = this._morse.split('').reverse().join('');
    }
    onClear() {
        this._morse = '';
        this.invalidateLookup();
    }
    onEmpty() {
        return this._morse.length === 0;
    }
    getEncoding() {
        return MorseCharacter.parseMorseString(this._morse);
    }
}
exports.MorseCharacter = MorseCharacter;
//# sourceMappingURL=MorseCharacter.js.map