"use strict";
// The encoding for morse is much less straightforward than the others. The dots
// and dashes each use two bits of the integer starting with the least-
// significant bit.
//
// Example:
//
// 'F' -> ..-.
//
// Which encodes as:
//
// 01 01 10 01
//  .  .  -  .
//
// Since it's packed starting with the LSB, the resulting number is reversed:
//
// 01100101 -> 0x65
//
Object.defineProperty(exports, "__esModule", { value: true });
exports.MorseEncoding = void 0;
function packMorse(...args) {
    let value = MorseEncoding.None;
    let shift = 0;
    for (let arg of args) {
        arg <<= shift;
        value |= arg;
        shift += 2;
    }
    return value;
}
var MorseEncoding;
(function (MorseEncoding) {
    MorseEncoding[MorseEncoding["None"] = 0] = "None";
    // Primitives
    MorseEncoding[MorseEncoding["Dot"] = 1] = "Dot";
    MorseEncoding[MorseEncoding["Dash"] = 2] = "Dash";
    // Letters
    MorseEncoding[MorseEncoding["LetterA"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dash)] = "LetterA";
    MorseEncoding[MorseEncoding["LetterB"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dot)] = "LetterB";
    MorseEncoding[MorseEncoding["LetterC"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dot)] = "LetterC";
    MorseEncoding[MorseEncoding["LetterD"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dot)] = "LetterD";
    MorseEncoding[MorseEncoding["LetterE"] = packMorse(MorseEncoding.Dot)] = "LetterE";
    MorseEncoding[MorseEncoding["LetterF"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dot)] = "LetterF";
    MorseEncoding[MorseEncoding["LetterG"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dot)] = "LetterG";
    MorseEncoding[MorseEncoding["LetterH"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dot)] = "LetterH";
    MorseEncoding[MorseEncoding["LetterI"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dot)] = "LetterI";
    MorseEncoding[MorseEncoding["LetterJ"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dash)] = "LetterJ";
    MorseEncoding[MorseEncoding["LetterK"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dash)] = "LetterK";
    MorseEncoding[MorseEncoding["LetterL"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dot)] = "LetterL";
    MorseEncoding[MorseEncoding["LetterM"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dash)] = "LetterM";
    MorseEncoding[MorseEncoding["LetterN"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dot)] = "LetterN";
    MorseEncoding[MorseEncoding["LetterO"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dash)] = "LetterO";
    MorseEncoding[MorseEncoding["LetterP"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dot)] = "LetterP";
    MorseEncoding[MorseEncoding["LetterQ"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dash)] = "LetterQ";
    MorseEncoding[MorseEncoding["LetterR"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dot)] = "LetterR";
    MorseEncoding[MorseEncoding["LetterS"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dot)] = "LetterS";
    MorseEncoding[MorseEncoding["LetterT"] = packMorse(MorseEncoding.Dash)] = "LetterT";
    MorseEncoding[MorseEncoding["LetterU"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dash)] = "LetterU";
    MorseEncoding[MorseEncoding["LetterV"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dash)] = "LetterV";
    MorseEncoding[MorseEncoding["LetterW"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dash)] = "LetterW";
    MorseEncoding[MorseEncoding["LetterX"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dash)] = "LetterX";
    MorseEncoding[MorseEncoding["LetterY"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dash)] = "LetterY";
    MorseEncoding[MorseEncoding["LetterZ"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dot)] = "LetterZ";
    // Numbers
    MorseEncoding[MorseEncoding["Number0"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dash)] = "Number0";
    MorseEncoding[MorseEncoding["Number1"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dash)] = "Number1";
    MorseEncoding[MorseEncoding["Number2"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dash)] = "Number2";
    MorseEncoding[MorseEncoding["Number3"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dash)] = "Number3";
    MorseEncoding[MorseEncoding["Number4"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dash)] = "Number4";
    MorseEncoding[MorseEncoding["Number5"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dot)] = "Number5";
    MorseEncoding[MorseEncoding["Number6"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dot)] = "Number6";
    MorseEncoding[MorseEncoding["Number7"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dot)] = "Number7";
    MorseEncoding[MorseEncoding["Number8"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dot)] = "Number8";
    MorseEncoding[MorseEncoding["Number9"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dot)] = "Number9";
    // Punctuation
    MorseEncoding[MorseEncoding["PunctuationPeriod"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dash)] = "PunctuationPeriod";
    MorseEncoding[MorseEncoding["PunctuationComma"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dash)] = "PunctuationComma";
    MorseEncoding[MorseEncoding["PunctuationQuestionMark"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dot)] = "PunctuationQuestionMark";
    MorseEncoding[MorseEncoding["PunctuationApostrophe"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dot)] = "PunctuationApostrophe";
    MorseEncoding[MorseEncoding["PunctuationExclamationPoint"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dash)] = "PunctuationExclamationPoint";
    MorseEncoding[MorseEncoding["PunctuationForwardSlash"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dot)] = "PunctuationForwardSlash";
    MorseEncoding[MorseEncoding["PunctuationOpenParenthesis"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dot)] = "PunctuationOpenParenthesis";
    MorseEncoding[MorseEncoding["PunctuationCloseParenthesis"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dash)] = "PunctuationCloseParenthesis";
    MorseEncoding[MorseEncoding["PunctuationAmpersand"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dot)] = "PunctuationAmpersand";
    MorseEncoding[MorseEncoding["PunctuationColon"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dot)] = "PunctuationColon";
    MorseEncoding[MorseEncoding["PunctuationSemicolon"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dot)] = "PunctuationSemicolon";
    MorseEncoding[MorseEncoding["PunctuationDoubleDash"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dash)] = "PunctuationDoubleDash";
    MorseEncoding[MorseEncoding["PunctuationPlusSign"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dot)] = "PunctuationPlusSign";
    MorseEncoding[MorseEncoding["PunctuationHyphen"] = packMorse(MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dash)] = "PunctuationHyphen";
    MorseEncoding[MorseEncoding["PunctuationUnderscore"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dash)] = "PunctuationUnderscore";
    MorseEncoding[MorseEncoding["PunctuationQuotationMark"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dot)] = "PunctuationQuotationMark";
    MorseEncoding[MorseEncoding["PunctuationDollarSign"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dot, MorseEncoding.Dash)] = "PunctuationDollarSign";
    MorseEncoding[MorseEncoding["PunctuationAtSign"] = packMorse(MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dash, MorseEncoding.Dot, MorseEncoding.Dash, MorseEncoding.Dot)] = "PunctuationAtSign";
})(MorseEncoding || (exports.MorseEncoding = MorseEncoding = {}));
//# sourceMappingURL=MorseEncoding.js.map