"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Resistor = void 0;
const ResistorColorEntry_1 = require("./ResistorColorEntry");
class Resistor {
    // Name, Color, Multiplier, Value, Tolerance
    static BLACK = new ResistorColorEntry_1.ResistorColorEntry('Black', '#000000', 1, 0);
    static BROWN = new ResistorColorEntry_1.ResistorColorEntry('Brown', '#8B4513', 10, 1, 1);
    static RED = new ResistorColorEntry_1.ResistorColorEntry('Red', '#FF0000', 100, 2, 2);
    static ORANGE = new ResistorColorEntry_1.ResistorColorEntry('Orange', '#EE7420', 1000, 3);
    static YELLOW = new ResistorColorEntry_1.ResistorColorEntry('Yellow', '#FFFF00', 10000, 4);
    static GREEN = new ResistorColorEntry_1.ResistorColorEntry('Green', '#008000', 100000, 5, 0.5);
    static BLUE = new ResistorColorEntry_1.ResistorColorEntry('Blue', '#0000FF', 1000000, 6, 0.25);
    static VIOLET = new ResistorColorEntry_1.ResistorColorEntry('Violet', '#800080', 10000000, 7, 0.1);
    static GREY = new ResistorColorEntry_1.ResistorColorEntry('Grey', '#808080', 100000000, 8);
    static WHITE = new ResistorColorEntry_1.ResistorColorEntry('White', '#FFFFFF', 1000000000, 9);
    static GOLD = new ResistorColorEntry_1.ResistorColorEntry('Gold', '#AB8D3F', 0.1, undefined, 5);
    static SILVER = new ResistorColorEntry_1.ResistorColorEntry('Silver', '#C0C0C0', 0.01, undefined, 10);
    static colorTable = [
        Resistor.BLACK,
        Resistor.BROWN,
        Resistor.RED,
        Resistor.ORANGE,
        Resistor.YELLOW,
        Resistor.GREEN,
        Resistor.BLUE,
        Resistor.VIOLET,
        Resistor.GREY,
        Resistor.WHITE,
        Resistor.GOLD,
        Resistor.SILVER,
    ];
    static INVALID_RESISTOR = -1;
    /**
     * getValue assumes there is no tolerance band, as tolerance is not part of
     * the value calculation
     */
    static getValue(colors) {
        if (colors.length < 3 || colors.length > 4) {
            throw new RangeError('Invalid resistor size');
        }
        // Should we throw an error if the first color is black?  In resistors, it's
        // technically not allowed, but in puzzle events who knows what they'll do.
        // The math works out fine (it's just a leading zero), so for now it's
        // allowed.
        // Iterate through all but the final band (the multipler) and extract the
        // values
        let value = 0;
        for (let i = 0; i < colors.length - 1; ++i) {
            const currentColor = colors[i];
            if (currentColor.value === undefined) {
                return Resistor.INVALID_RESISTOR;
            }
            value *= 10;
            value += currentColor.value;
        }
        return Resistor.applyMultiplier(value, colors[colors.length - 1].multiplier);
    }
    static getDisplayValue(value) {
        if (value >= 1000000000) {
            return `${value / 1000000000}G`;
        }
        else if (value >= 1000000) {
            return `${value / 1000000}M`;
        }
        else if (value >= 1000) {
            return `${value / 1000}k`;
        }
        else {
            return value.toString();
        }
    }
    static applyMultiplier(value, multipler) {
        if (multipler >= 1) {
            return value * multipler;
        }
        else if (multipler > 0) {
            // Avoid floating point multiplication issues
            return value / (1 / multipler);
        }
        else {
            throw new RangeError('Invalid multiplier');
        }
    }
}
exports.Resistor = Resistor;
//# sourceMappingURL=Resistor.js.map