"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SemaphoreCharacter = void 0;
const EncodingCategory_1 = require("../Common/EncodingCategory");
const EncodingCharacterBase_1 = require("../Common/EncodingCharacterBase");
const SemaphoreData_1 = require("./SemaphoreData");
const SemaphoreDegrees_1 = require("./SemaphoreDegrees");
const SemaphoreEncoding_1 = require("./SemaphoreEncoding");
class SemaphoreCharacter extends EncodingCharacterBase_1.EncodingCharacterBase {
    static parseEncoding(encoding) {
        const directions = [];
        for (let i = 1; i <= 8; i++) {
            const direction = (1 << i);
            if ((encoding & direction) === direction) {
                directions.push(direction);
            }
        }
        return directions;
    }
    _directions = [];
    constructor(encoding = SemaphoreEncoding_1.SemaphoreEncoding.None, category = EncodingCategory_1.EncodingCategory.All) {
        super(SemaphoreData_1.SemaphoreData.instance, category);
        this.addDirection(encoding);
    }
    get directions() {
        return this._directions;
    }
    set directions(value) {
        this._directions = value.slice(0, 2);
        this.invalidateLookup();
    }
    addDirection(direction) {
        for (const dir of SemaphoreCharacter.parseEncoding(direction)) {
            this._directions.push(dir);
        }
        if (this._directions.length > 2) {
            this._directions.splice(0, this._directions.length - 2);
        }
        this.invalidateLookup();
    }
    removeDirection(direction) {
        for (const dir of SemaphoreCharacter.parseEncoding(direction)) {
            const index = this._directions.indexOf(dir);
            if (index >= 0) {
                this._directions.splice(index, 1);
            }
        }
        this.invalidateLookup();
    }
    hasDirection(direction) {
        return this._directions.indexOf(direction) >= 0;
    }
    getDegrees() {
        return this._directions
            .map(value => SemaphoreDegrees_1.SemaphoreDegrees.ToDegrees(value))
            .sort((a, b) => a - b);
    }
    getPotentialMatch(other) {
        if (this.directions.length !== 1 || this.directions[0] === other) {
            return null;
        }
        const potentialMatch = this.getPotentialMatches().filter(value => (value.encoding & other) === other)[0];
        return potentialMatch || null;
    }
    onClear() {
        this._directions.length = 0;
    }
    onEmpty() {
        return this._directions.length === 0;
    }
    getEncoding() {
        return this._directions.reduce((previousValue, currentValue) => (previousValue |= currentValue), SemaphoreEncoding_1.SemaphoreEncoding.None);
    }
}
exports.SemaphoreCharacter = SemaphoreCharacter;
//# sourceMappingURL=SemaphoreCharacter.js.map