"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SemaphoreStream = void 0;
const EncodingCategory_1 = require("../Common/EncodingCategory");
const SemaphoreData_1 = require("./SemaphoreData");
const SemaphoreEncoding_1 = require("./SemaphoreEncoding");
class SemaphoreStream {
    _chars = [];
    _state;
    _currentStr = '';
    _processPosition = 0;
    constructor(chars) {
        if (chars) {
            this._chars = Array.from(chars);
        }
        this._state = {
            numberMode: false,
        };
        this.invalidate();
    }
    get chars() {
        return this._chars;
    }
    set chars(value) {
        this.clear();
        for (const ch of value) {
            this._chars.push(ch);
        }
    }
    append(ch) {
        this._chars.push(ch.valueOf());
    }
    clear() {
        this._chars.length = 0;
        this.invalidate();
    }
    space() {
        this._chars.push(SemaphoreEncoding_1.SemaphoreEncoding.None);
    }
    toString() {
        this.update();
        return this._currentStr;
    }
    invalidate() {
        this._currentStr = '';
        this._processPosition = 0;
    }
    update() {
        while (this._processPosition < this._chars.length) {
            const ch = this._chars[this._processPosition];
            if (ch === SemaphoreEncoding_1.SemaphoreEncoding.None) {
                this._state.numberMode = false;
                this._currentStr += ' ';
            }
            else if (ch === SemaphoreEncoding_1.SemaphoreEncoding.FormattingNumber) {
                this._state.numberMode = true;
                this._currentStr += '#';
            }
            else {
                const category = EncodingCategory_1.EncodingCategory.Punctuation |
                    (this._state.numberMode
                        ? EncodingCategory_1.EncodingCategory.Number
                        : EncodingCategory_1.EncodingCategory.Letter);
                const exact = SemaphoreData_1.SemaphoreData.instance.lookup(ch, category).exact;
                if (exact.length > 0) {
                    this._currentStr += exact[0].toString();
                }
            }
            this._processPosition++;
        }
    }
}
exports.SemaphoreStream = SemaphoreStream;
//# sourceMappingURL=SemaphoreStream.js.map